@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hcrp.c,v 1.2 2004/12/25 17:43:16 holtmann Exp $
+ *  $Id: hcrp.c,v 1.3 2005/07/05 21:15:41 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -176,8 +176,9 @@ int hcrp_print(bdaddr_t *src, bdaddr_t *dst, unsigned short ctrl_psm, unsigned s
 {
 	struct sockaddr_l2 addr;
 	struct l2cap_options opts;
+	socklen_t size;
 	unsigned char buf[2048];
-	int i, size, ctrl_sk, data_sk, mtu, count, len, timeout = 0;
+	int i, ctrl_sk, data_sk, mtu, count, len, timeout = 0;
 	uint8_t status;
 	uint16_t tid = 0;
 	uint32_t tmp, credit = 0;
