@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,15 +24,20 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: dund.h,v 1.3 2004/11/11 18:25:46 holtmann Exp $
+ *  $Id: dund.h,v 1.5 2005/08/21 08:56:10 holtmann Exp $
  */
 
 /* DUN scripts & commands */
-#define DUN_CONFIG_DIR      "/etc/bluetooth/dun"
+#define DUN_CONFIG_DIR	"/etc/bluetooth/dun"
 
-#define DUN_DEFAULT_CHANNEL 1
+#define DUN_DEFAULT_CHANNEL	1
 
-#define DUN_MAX_PPP_OPTS    40
+#define DUN_MAX_PPP_OPTS	40
+
+/* DUN types */
+#define LANACCESS	0
+#define MROUTER		1
+#define ACTIVESYNC	2
 
 /* DUN functions */
 int dun_init(void);
@@ -45,6 +50,6 @@ int dun_kill_all_connections(void);
 int dun_open_connection(int sk, char *pppd, char **pppd_opts, int wait);
 
 /* SDP functions */
-int  dun_sdp_register(uint8_t channel, int mrouter);
+int  dun_sdp_register(bdaddr_t *device, uint8_t channel, int type);
 void dun_sdp_unregister(void);
-int  dun_sdp_search(bdaddr_t *src, bdaddr_t *dst, int *channel, int mrouter);
+int  dun_sdp_search(bdaddr_t *src, bdaddr_t *dst, int *channel, int type);
