@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: attest.c,v 1.3 2005/01/27 20:30:48 holtmann Exp $
+ *  $Id: attest.c,v 1.4 2005/07/05 21:15:51 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,7 +47,7 @@ static int at_command(int fd, char *cmd, int to)
 {
 	fd_set rfds;
 	struct timeval timeout;
-	unsigned char buf[1024];
+	char buf[1024];
 	int sel, len, i, n;
 
 	write(fd, cmd, strlen(cmd));
