@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hstest.c,v 1.6 2004/12/25 17:43:20 holtmann Exp $
+ *  $Id: hstest.c,v 1.7 2005/07/05 21:15:51 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -89,7 +89,8 @@ static int sco_connect(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t
 	struct sockaddr_sco addr;
 	struct sco_conninfo conn;
 	struct sco_options opts;
-	int s, size;
+	socklen_t size;
+	int s;
 
 	if ((s = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_SCO)) < 0) {
 		return -1;
