@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hciconfig.c,v 1.53 2005/01/31 05:45:39 holtmann Exp $
+ *  $Id: hciconfig.c,v 1.69 2005/09/10 00:34:21 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,7 @@
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 
+#include "textfile.h"
 #include "csr.h"
 
 static struct hci_dev_info di;
@@ -75,6 +76,12 @@ static void print_dev_list(int ctl, int flags)
 		di.dev_id = (dr+i)->dev_id;
 		if (ioctl(ctl, HCIGETDEVINFO, (void *) &di) < 0)
 			continue;
+		if (hci_test_bit(HCI_RAW, &di.flags) &&
+				!bacmp(&di.bdaddr, BDADDR_ANY)) {
+			int dd = hci_open_dev(di.dev_id);
+			hci_read_bd_addr(dd, &di.bdaddr, 1000);
+			hci_close_dev(dd);
+		}
 		print_dev_info(ctl, &di);
 	}
 }
@@ -96,19 +103,16 @@ static void print_link_mode(struct hci_dev_info *di)
 
 static void print_dev_features(struct hci_dev_info *di, int format)
 {
-	if (!format) {
-		printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n", 
-			di->features[0], di->features[1],
-			di->features[2], di->features[3],
-			di->features[4], di->features[5],
-			di->features[6], di->features[7] );
-	} else {
-		printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n%s\n", 
-			di->features[0], di->features[1],
-			di->features[2], di->features[3],
-			di->features[4], di->features[5],
-			di->features[6], di->features[7],
-			lmp_featurestostr(di->features, "\t\t", 63));
+	printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x "
+				"0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n",
+		di->features[0], di->features[1], di->features[2],
+		di->features[3], di->features[4], di->features[5],
+		di->features[6], di->features[7]);
+
+	if (format) {
+		char *tmp = lmp_featurestostr(di->features, "\t\t", 63);
+		printf("%s\n", tmp);
+		bt_free(tmp);
 	}
 }
 
@@ -377,8 +381,50 @@ static void cmd_scomtu(int ctl, int hdev, char *opt)
 
 static void cmd_features(int ctl, int hdev, char *opt)
 {
+	uint8_t max_page, features[8];
+	char *tmp;
+	int i, dd;
+
+	if (!(di.features[7] & LMP_EXT_FEAT)) {
+		print_dev_hdr(&di);
+		print_dev_features(&di, 1);
+		return;
+	}
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (hci_read_local_ext_features(dd, 0, &max_page, features, 1000) < 0) {
+		fprintf(stderr, "Can't read extended features hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
 	print_dev_hdr(&di);
-	print_dev_features(&di, 1);
+	tmp = lmp_featurestostr(di.features, "\t\t", 63);
+	printf("\tFeatures%s: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x "
+				"0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n",
+		(max_page > 0) ? " page 0" : "",
+		features[0], features[1], features[2], features[3],
+		features[4], features[5], features[6], features[7]);
+	printf("%s\n", tmp);
+	bt_free(tmp);
+
+	for (i = 1; i <= max_page; i++) {
+		if (hci_read_local_ext_features(dd, 1, &max_page, features, 1000) < 0)
+			continue;
+
+		printf("\tFeatures page %d: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x "
+					"0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n", i,
+			features[0], features[1], features[2], features[3],
+			features[4], features[5], features[6], features[7]);
+	}
+
+	hci_close_dev(dd);
 }
 
 static void cmd_name(int ctl, int hdev, char *opt)
@@ -397,7 +443,7 @@ static void cmd_name(int ctl, int hdev, char *opt)
 			exit(1);
 		}
 	} else {
-		char name[248];
+		char name[249];
 		int i;
 		if (hci_read_local_name(s, sizeof(name), name, 1000) < 0) {
 			fprintf(stderr, "Can't read local name on hci%d: %s (%d)\n",
@@ -407,6 +453,7 @@ static void cmd_name(int ctl, int hdev, char *opt)
 		for (i = 0; i < 248 && name[i]; i++)
 			if (!isprint(name[i]))
 				name[i] = '.';
+		name[248] = '\0';
 		print_dev_hdr(&di);
 		printf("\tName: '%s'\n", name);
 	}
@@ -602,7 +649,7 @@ static void cmd_class(int ctl, int hdev, char *opt)
 		if ((cls[1] & 0x1f) > sizeof(*major_devices))
 			printf("Invalid Device Class!\n");
 		else
-			printf("%s, %s\n", major_devices[cls[1] & 0x1f], 
+			printf("%s, %s\n", major_devices[cls[1] & 0x1f],
 				get_minor_device_name(cls[1] & 0x1f, cls[0] >> 2));
 	}
 }
@@ -650,9 +697,144 @@ static void cmd_voice(int ctl, int hdev, char *opt)
 	}
 }
 
+static int get_link_key(const bdaddr_t *local, const bdaddr_t *peer, uint8_t *key)
+{
+	char filename[PATH_MAX + 1], addr[18], tmp[3], *str;
+	int i;
+
+	ba2str(local, addr);
+	snprintf(filename, PATH_MAX, "%s/%s/linkkeys", STORAGEDIR, addr);
+
+	ba2str(peer, addr);
+	str = textfile_get(filename, addr);
+	if (!str)
+		return -EIO;
+
+	memset(tmp, 0, sizeof(tmp));
+	for (i = 0; i < 16; i++) {
+		memcpy(tmp, str + (i * 2), 2);
+		key[i] = (uint8_t) strtol(tmp, NULL, 16);
+	}
+
+	free(str);
+
+	return 0;
+}
+
+static void cmd_putkey(int ctl, int hdev, char *opt)
+{
+	struct hci_dev_info di;
+	bdaddr_t bdaddr;
+	uint8_t key[16];
+	int dd;
+
+	if (!opt)
+		return;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (hci_devinfo(hdev, &di) < 0) {
+		fprintf(stderr, "Can't get device info for hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	str2ba(opt, &bdaddr);
+	if (get_link_key(&di.bdaddr, &bdaddr, key) < 0) {
+		fprintf(stderr, "Can't find link key for %s on hci%d\n", opt, hdev);
+		exit(1);
+	}
+
+	if (hci_write_stored_link_key(dd, &bdaddr, key, 1000) < 0) {
+		fprintf(stderr, "Can't write stored link key on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	hci_close_dev(dd);
+}
+
+static void cmd_delkey(int ctl, int hdev, char *opt)
+{
+	bdaddr_t bdaddr;
+	uint8_t all;
+	int dd;
+
+	if (!opt)
+		return;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (!strcasecmp(opt, "all")) {
+		bacpy(&bdaddr, BDADDR_ANY);
+		all = 1;
+	} else {
+		str2ba(opt, &bdaddr);
+		all = 0;
+	}
+
+	if (hci_delete_stored_link_key(dd, &bdaddr, all, 1000) < 0) {
+		fprintf(stderr, "Can't delete stored link key on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	hci_close_dev(dd);
+}
+
+static void cmd_commands(int ctl, int hdev, char *opt)
+{
+	uint8_t cmds[64];
+	char *str;
+	int i, n, dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (hci_read_local_commands(dd, cmds, 1000) < 0) {
+		fprintf(stderr, "Can't read support commands on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	print_dev_hdr(&di);
+	for (i = 0; i < 64; i++) {
+		if (!cmds[i])
+			continue;
+
+		printf("%s Octet %-2d = 0x%02x (Bit",
+			i ? "\t\t ": "\tCommands:", i, cmds[i]);
+		for (n = 0; n < 8; n++)
+			if (hci_test_bit(n, &cmds[i]))
+				printf(" %d", n);
+		printf(")\n");
+	}
+
+	str = hci_commandstostr(cmds, "\t", 71);
+	printf("%s\n", str);
+	bt_free(str);
+
+	hci_close_dev(dd);
+}
+
 static void cmd_version(int ctl, int hdev, char *opt)
 {
 	struct hci_version ver;
+	char *hciver, *lmpver;
 	int dd;
 
 	dd = hci_open_dev(hdev);
@@ -668,12 +850,22 @@ static void cmd_version(int ctl, int hdev, char *opt)
 		exit(1);
 	}
 
+	hciver = hci_vertostr(ver.hci_ver);
+	lmpver = lmp_vertostr(ver.hci_ver);
+
 	print_dev_hdr(&di);
 	printf("\tHCI Ver: %s (0x%x) HCI Rev: 0x%x LMP Ver: %s (0x%x) LMP Subver: 0x%x\n"
 		"\tManufacturer: %s (%d)\n",
-		hci_vertostr(ver.hci_ver), ver.hci_ver, ver.hci_rev,
-		lmp_vertostr(ver.lmp_ver), ver.lmp_ver, ver.lmp_subver,
+		hciver ? hciver : "n/a", ver.hci_ver, ver.hci_rev,
+		lmpver ? lmpver : "n/a", ver.lmp_ver, ver.lmp_subver,
 		bt_compidtostr(ver.manufacturer), ver.manufacturer);
+
+	if (hciver)
+		bt_free(hciver);
+	if (lmpver)
+		bt_free(lmpver);
+
+	hci_close_dev(dd);
 }
 
 static void cmd_inq_mode(int ctl, int hdev, char *opt)
@@ -705,8 +897,55 @@ static void cmd_inq_mode(int ctl, int hdev, char *opt)
 		}
 
 		print_dev_hdr(&di);
-		printf("\tInquiry mode: %s\n",
-			mode == 1 ? "Inquiry with RSSI" : "Standard Inquiry");
+		printf("\tInquiry mode: ");
+		switch (mode) {
+		case 0:
+			printf("Standard Inquiry\n");
+			break;
+		case 1:
+			printf("Inquiry with RSSI\n");
+			break;
+		case 2:
+			printf("Inquiry with RSSI or Extended Inquiry\n");
+			break;
+		default:
+			printf("Unknown (0x%02x)\n", mode);
+			break;
+		}
+	}
+}
+
+static void cmd_inq_type(int ctl, int hdev, char *opt)
+{
+	int dd;
+
+	dd = hci_open_dev(hdev);
+	if (dd < 0) {
+		fprintf(stderr, "Can't open device hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+		exit(1);
+	}
+
+	if (opt) {
+		uint8_t type = atoi(opt);
+
+		if (hci_write_inquiry_scan_type(dd, type, 2000) < 0) {
+			fprintf(stderr, "Can't set inquiry scan type on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+	} else {
+		uint8_t type;
+
+		if (hci_read_inquiry_scan_type(dd, &type, 1000) < 0) {
+			fprintf(stderr, "Can't read inquiry scan type on hci%d: %s (%d)\n",
+						hdev, strerror(errno), errno);
+			exit(1);
+		}
+
+		print_dev_hdr(&di);
+		printf("\tInquiry scan type: %s\n",
+			type == 1 ? "Interlaced Inquiry Scan" : "Standard Inquiry Scan");
 	}
 }
 
@@ -737,8 +976,8 @@ static void cmd_inq_parms(int ctl, int hdev, char *opt)
 		rq.cparam = &cp;
 		rq.clen = WRITE_INQ_ACTIVITY_CP_SIZE;
 
-		cp.window = htobs((uint16_t)window);
-		cp.interval = htobs((uint16_t)interval);
+		cp.window = htobs((uint16_t) window);
+		cp.interval = htobs((uint16_t) interval);
 
 		if (window < 0x12 || window > 0x1000)
 			printf("Warning: inquiry window out of range!\n");
@@ -806,8 +1045,8 @@ static void cmd_page_parms(int ctl, int hdev, char *opt)
 		rq.cparam = &cp;
 		rq.clen = WRITE_PAGE_ACTIVITY_CP_SIZE;
 
-		cp.window = htobs((uint16_t)window);
-		cp.interval = htobs((uint16_t)interval);
+		cp.window = htobs((uint16_t) window);
+		cp.interval = htobs((uint16_t) interval);
 
 		if (window < 0x12 || window > 0x1000)
 			printf("Warning: page window out of range!\n");
@@ -875,7 +1114,7 @@ static void cmd_page_to(int ctl, int hdev, char *opt)
 		rq.cparam = &cp;
 		rq.clen = WRITE_PAGE_TIMEOUT_CP_SIZE;
 
-		cp.timeout = htobs((uint16_t)timeout);
+		cp.timeout = htobs((uint16_t) timeout);
 
 		if (timeout < 0x01 || timeout > 0xFFFF)
 			printf("Warning: page timeout out of range!\n");
@@ -986,7 +1225,7 @@ static void print_rev_csr(int dd, uint16_t rev)
 	if (!csr_read_varid_uint16(dd, 3, CSR_VARID_MAX_CRYPT_KEY_LENGTH, &maxkeylen))
 		printf("\tMax key size: %d bit\n", maxkeylen * 8);
 
-	if (!csr_read_pskey_uint16(dd, 4, CSR_PSKEY_HOSTIO_MAP_SCO_PCM, &mapsco))
+	if (!csr_read_pskey_uint16(dd, 4, CSR_PSKEY_HOSTIO_MAP_SCO_PCM, 0x0000, &mapsco))
 		printf("\tSCO mapping:  %s\n", mapsco ? "PCM" : "HCI");
 }
 
@@ -1014,7 +1253,7 @@ static void print_rev_digianswer(int dd)
 
 static void print_rev_broadcom(uint16_t hci_rev, uint16_t lmp_subver)
 {
-	printf("\tFirmware %d.%d.%03d\n", hci_rev, lmp_subver >> 8, lmp_subver & 0xff);
+	printf("\tFirmware %d.%d / %d\n", hci_rev, lmp_subver >> 8, lmp_subver & 0xff);
 }
 
 static void print_rev_avm(uint16_t hci_rev, uint16_t lmp_subver)
@@ -1144,12 +1383,16 @@ static struct {
 	{ "voice",	cmd_voice,	"[voice]",	"Get/Set voice setting" },
 	{ "iac",	cmd_iac,	"[iac]",	"Get/Set inquiry access code" },
 	{ "inqmode",	cmd_inq_mode,	"[mode]",	"Get/set inquiry mode" },
+	{ "inqtype",	cmd_inq_type,	"[type]",	"Get/set inquiry scan type" },
 	{ "inqparms",	cmd_inq_parms,	"[win:int]",	"Get/Set inquiry scan window and interval" },
 	{ "pageparms",	cmd_page_parms,	"[win:int]",	"Get/Set page scan window and interval" },
 	{ "pageto",	cmd_page_to,	"[to]",		"Get/Set page timeout" },
 	{ "afhmode",	cmd_afh_mode,	"[mode]",	"Get/Set AFH mode" },
 	{ "aclmtu",	cmd_aclmtu,	"<mtu:pkt>",	"Set ACL MTU and number of packets" },
 	{ "scomtu",	cmd_scomtu,	"<mtu:pkt>",	"Set SCO MTU and number of packets" },
+	{ "putkey",	cmd_putkey,	"<bdaddr>",	"Store link key on the device" },
+	{ "delkey",	cmd_delkey,	"<bdaddr>",	"Delete link key from the device" },
+	{ "commands",	cmd_commands,	0,		"Display supported commands" },
 	{ "features",	cmd_features,	0,		"Display device features" },
 	{ "version",	cmd_version,	0,		"Display version information" },
 	{ "revision",	cmd_revision,	0,		"Display revision information" },
@@ -1217,6 +1460,13 @@ int main(int argc, char **argv, char **env)
 		exit(1);
 	}
 
+	if (hci_test_bit(HCI_RAW, &di.flags) &&
+			!bacmp(&di.bdaddr, BDADDR_ANY)) {
+		int dd = hci_open_dev(di.dev_id);
+		hci_read_bd_addr(dd, &di.bdaddr, 1000);
+		hci_close_dev(dd);
+	}
+
 	while (argc > 0) {
 		for (i = 0; command[i].cmd; i++) {
 			if (strncmp(command[i].cmd, *argv, 5))
