@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hcitool.c,v 1.56 2005/01/25 21:41:41 holtmann Exp $
+ *  $Id: hcitool.c,v 1.80 2005/09/07 16:15:39 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,9 +34,12 @@
 
 #include <stdio.h>
 #include <errno.h>
+#include <ctype.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <getopt.h>
+#include <sys/param.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 
@@ -44,6 +47,9 @@
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 
+#include "textfile.h"
+#include "oui.h"
+
 #define for_each_opt(opt, long, short) while ((opt=getopt_long(argc, argv, short ? short:"+", long, NULL)) != -1)
 
 static void usage(void);
@@ -140,6 +146,151 @@ static void hex_dump(char *pref, int width, unsigned char *buf, int len)
 		printf("\n");
 }
 
+static char *get_minor_device_name(int major, int minor)
+{
+	switch (major) {
+	case 0:	/* misc */
+		return "";
+	case 1:	/* computer */
+		switch(minor) {
+		case 0:
+			return "Uncategorized";
+		case 1:
+			return "Desktop workstation";
+		case 2:
+			return "Server";
+		case 3:
+			return "Laptop";
+		case 4:
+			return "Handheld";
+		case 5:
+			return "Palm";
+		case 6:
+			return "Wearable";
+		}
+		break;
+	case 2:	/* phone */
+		switch(minor) {
+		case 0:
+			return "Uncategorized";
+		case 1:
+			return "Cellular";
+		case 2:
+			return "Cordless";
+		case 3:
+			return "Smart phone";
+		case 4:
+			return "Wired modem or voice gateway";
+		case 5:
+			return "Common ISDN Access";
+		case 6:
+			return "Sim Card Reader";
+		}
+		break;
+	case 3:	/* lan access */
+		if (minor == 0)
+			return "Uncategorized";
+		switch(minor / 8) {
+		case 0:
+			return "Fully available";
+		case 1:
+			return "1-17% utilized";
+		case 2:
+			return "17-33% utilized";
+		case 3:
+			return "33-50% utilized";
+		case 4:
+			return "50-67% utilized";
+		case 5:
+			return "67-83% utilized";
+		case 6:
+			return "83-99% utilized";
+		case 7:
+			return "No service available";
+		}
+		break;
+	case 4:	/* audio/video */
+		switch(minor) {
+		case 0:
+			return "Uncategorized";
+		case 1:
+			return "Device conforms to the Headset profile";
+		case 2:
+			return "Hands-free";
+			/* 3 is reserved */
+		case 4:
+			return "Microphone";
+		case 5:
+			return "Loudspeaker";
+		case 6:
+			return "Headphones";
+		case 7:
+			return "Portable Audio";
+		case 8:
+			return "Car Audio";
+		case 9:
+			return "Set-top box";
+		case 10:
+			return "HiFi Audio Device";
+		case 11:
+			return "VCR";
+		case 12:
+			return "Video Camera";
+		case 13:
+			return "Camcorder";
+		case 14:
+			return "Video Monitor";
+		case 15:
+			return "Video Display and Loudspeaker";
+		case 16:
+			return "Video Conferencing";
+			/* 17 is reserved */
+		case 18:
+			return "Gaming/Toy";
+		}
+		break;
+	case 5:	/* peripheral */
+		switch(minor) {
+		case 16:
+			return "Keyboard";
+		case 32:
+			return "Pointing device";
+		case 48:
+			return "Combo keyboard/pointing device";
+		}
+		break;
+	case 6:	/* imaging */
+		if (minor & 4)
+			return "Display";
+		if (minor & 8)
+			return "Camera";
+		if (minor & 16)
+			return "Scanner";
+		if (minor & 32)
+			return "Printer";
+		break;
+	case 63:	/* uncategorised */
+		return "";
+	}
+	return "Unknown (reserved) minor device class";
+}
+
+static char *major_classes[] = {
+	"Miscellaneous", "Computer", "Phone", "LAN Access",
+	"Audio/Video", "Peripheral", "Imaging", "Uncategorized"
+};
+
+static char *get_device_name(const bdaddr_t *local, const bdaddr_t *peer)
+{
+	char filename[PATH_MAX + 1], addr[18];
+
+	ba2str(local, addr);
+	snprintf(filename, PATH_MAX, "%s/%s/names", STORAGEDIR, addr);
+
+	ba2str(peer, addr);
+	return textfile_get(filename, addr);
+}
+
 /* Display local devices */
 
 static struct option dev_options[] = {
@@ -190,7 +341,7 @@ static void cmd_inq(int dev_id, int argc, char **argv)
 	int i, opt;
 
 	length  = 8;	/* ~10 seconds */
-	num_rsp = 100;
+	num_rsp = 0;
 	flags   = 0;
 
 	for_each_opt(opt, inq_options, NULL) {
@@ -223,12 +374,13 @@ static void cmd_inq(int dev_id, int argc, char **argv)
 	for (i = 0; i < num_rsp; i++) {
 		ba2str(&(info+i)->bdaddr, addr);
 		printf("\t%s\tclock offset: 0x%4.4x\tclass: 0x%2.2x%2.2x%2.2x\n",
-			addr, (info+i)->clock_offset,
-			(info+i)->dev_class[2], 
-			(info+i)->dev_class[1], 
+			addr, btohs((info+i)->clock_offset),
+			(info+i)->dev_class[2],
+			(info+i)->dev_class[1],
 			(info+i)->dev_class[0]);
 	}
-	free(info);
+
+	bt_free(info);
 }
 
 /* Device scanning */
@@ -238,23 +390,33 @@ static struct option scan_options[] = {
 	{ "length",	1, 0, 'l' },
 	{ "numrsp",	1, 0, 'n' },
 	{ "flush",	0, 0, 'f' },
+	{ "class",	0, 0, 'c' },
+	{ "info",	0, 0, 'i' },
+	{ "oui",	0, 0, 'o' },
+	{ "all",	0, 0, 'a' },
+	{ "ext",	0, 0, 'a' },
 	{ 0, 0, 0, 0 }
 };
 
 static char *scan_help =
 	"Usage:\n"
-	"\tscan [--length=N] [--numrsp=N] [--flush]\n";
+	"\tscan [--length=N] [--numrsp=N] [--flush] [--class] [--info] [--oui]\n";
 
 static void cmd_scan(int dev_id, int argc, char **argv)
 {
 	inquiry_info *info = NULL;
 	int num_rsp, length, flags;
-	char addr[18];
-	char name[248];
-	int i, opt, dd;
+	uint8_t cls[3], features[8];
+	uint16_t handle;
+	char addr[18], name[249], oui[9], *comp, *tmp;
+	struct hci_version version;
+	struct hci_dev_info di;
+	struct hci_conn_info_req *cr;
+	int extcls = 0, extinf = 0, extoui = 0;
+	int i, n, opt, dd, cc, nc;
 
 	length  = 8;	/* ~10 seconds */
-	num_rsp = 100;
+	num_rsp = 0;
 	flags   = 0;
 
 	for_each_opt(opt, scan_options, NULL) {
@@ -271,6 +433,24 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 			flags |= IREQ_CACHE_FLUSH;
 			break;
 
+		case 'c':
+			extcls = 1;
+			break;
+
+		case 'i':
+			extinf = 1;
+			break;
+
+		case 'o':
+			extoui = 1;
+			break;
+
+		case 'a':
+			extcls = 1;
+			extinf = 1;
+			extoui = 1;
+			break;
+
 		default:
 			printf(scan_help);
 			return;
@@ -285,6 +465,11 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 		}
 	}
 
+	if (hci_devinfo(dev_id, &di) < 0) {
+		perror("Can't get device info");
+		exit(1);
+	}
+
 	printf("Scanning ...\n");
 	num_rsp = hci_inquiry(dev_id, length, num_rsp, NULL, &info, flags);
 	if (num_rsp < 0) {
@@ -299,16 +484,151 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
+	if (extcls || extinf || extoui)
+		printf("\n");
+
 	for (i = 0; i < num_rsp; i++) {
 		memset(name, 0, sizeof(name));
-		if (hci_read_remote_name(dd, &(info+i)->bdaddr, sizeof(name), name, 100000) < 0)
-			strcpy(name, "n/a");
+		tmp = get_device_name(&di.bdaddr, &(info+i)->bdaddr);
+		if (tmp) {
+			strncpy(name, tmp, 249);
+			free(tmp);
+			nc = 1;
+		} else
+			nc = 0;
+
+		if (!extcls && !extinf && !extoui) {
+			ba2str(&(info+i)->bdaddr, addr);
+
+			if (nc) {
+				printf("\t%s\t%s\n", addr, name);
+				continue;
+			}
+
+			if (hci_read_remote_name_with_clock_offset(dd,
+					&(info+i)->bdaddr,
+					(info+i)->pscan_rep_mode,
+					(info+i)->clock_offset | 0x8000,
+					sizeof(name), name, 100000) < 0)
+				strcpy(name, "n/a");
+
+			for (n = 0; n < 248 && name[n]; n++)
+				if (!isprint(name[n]))
+					name[n] = '.';
+			name[248] = '\0';
+
+			printf("\t%s\t%s\n", addr, name);
+			continue;
+		}
+
 		ba2str(&(info+i)->bdaddr, addr);
-		printf("\t%s\t%s\n", addr, name);
+		printf("BD Address:\t%s [mode %d, clkoffset 0x%4.4x]\n", addr,
+			(info+i)->pscan_rep_mode, btohs((info+i)->clock_offset));
+
+		if (extoui) {
+			ba2oui(&(info+i)->bdaddr, oui);
+			comp = ouitocomp(oui);
+			if (comp) {
+				printf("OUI company:\t%s (%s)\n", comp, oui);
+				free(comp);
+			}
+		}
+
+		cc = 0;
+
+		if (extinf) {
+			cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+			if (cr) {
+				bacpy(&cr->bdaddr, &(info+i)->bdaddr);
+				cr->type = ACL_LINK;
+				if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+					handle = 0;
+					cc = 1;
+				} else {
+					handle = htobs(cr->conn_info->handle);
+					cc = 0;
+				}
+				free(cr);
+			}
+
+			if (cc) {
+				if (hci_create_connection(dd, &(info+i)->bdaddr,
+						htobs(di.pkt_type & ACL_PTYPE_MASK),
+						(info+i)->clock_offset | 0x8000,
+						0x01, &handle, 25000) < 0) {
+					handle = 0;
+					cc = 0;
+				}
+			}
+		}
+
+		if (handle > 0 || !nc) {
+			if (hci_read_remote_name_with_clock_offset(dd,
+					&(info+i)->bdaddr,
+					(info+i)->pscan_rep_mode,
+					(info+i)->clock_offset | 0x8000,
+					sizeof(name), name, 100000) < 0) {
+				if (!nc)
+					strcpy(name, "n/a");
+			} else {
+				for (n = 0; n < 248 && name[n]; n++)
+					if (!isprint(name[n]))
+						name[n] = '.';
+				name[248] = '\0';
+				nc = 0;
+			}
+		}
+
+		if (strlen(name) > 0)
+			printf("Device name:\t%s%s\n", name, nc ? " [cached]" : "");
+
+		if (extcls) {
+			memcpy(cls, (info+i)->dev_class, 3);
+			printf("Device class:\t");
+			if ((cls[1] & 0x1f) > sizeof(*major_classes))
+				printf("Invalid");
+			else
+				printf("%s, %s", major_classes[cls[1] & 0x1f],
+					get_minor_device_name(cls[1] & 0x1f, cls[0] >> 2));
+			printf(" (0x%2.2x%2.2x%2.2x)\n", cls[2], cls[1], cls[0]);
+		}
+
+		if (extinf && handle > 0) {
+			if (hci_read_remote_version(dd, handle, &version, 20000) == 0) {
+				char *ver = lmp_vertostr(version.lmp_ver);
+				printf("Manufacturer:\t%s (%d)\n",
+					bt_compidtostr(version.manufacturer),
+					version.manufacturer);
+				printf("LMP version:\t%s (0x%x) [subver 0x%x]\n",
+					ver ? ver : "n/a",
+					version.lmp_ver, version.lmp_subver);
+				if (ver)
+					bt_free(ver);
+			}
+
+			if (hci_read_remote_features(dd, handle, features, 20000) == 0) {
+				char *tmp = lmp_featurestostr(features, "\t\t", 63);
+				printf("LMP features:\t0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x"
+					" 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n",
+					features[0], features[1],
+					features[2], features[3],
+					features[4], features[5],
+					features[6], features[7]);
+				printf("%s\n", tmp);
+				bt_free(tmp);
+			}
+
+			if (cc) {
+				usleep(10000);
+				hci_disconnect(dd, handle, HCI_OE_USER_ENDED_CONNECTION, 10000);
+			}
+		}
+
+		printf("\n");
 	}
 
 	close(dd);
-	free(info);
+	bt_free(info);
 }
 
 /* Remote name */
@@ -380,8 +700,8 @@ static void cmd_info(int dev_id, int argc, char **argv)
 {
 	bdaddr_t bdaddr;
 	uint16_t handle;
-	char name[248];
-	unsigned char features[8];
+	uint8_t max_page, features[8];
+	char name[249], oui[9], *comp;
 	struct hci_version version;
 	struct hci_dev_info di;
 	struct hci_conn_info_req *cr;
@@ -451,24 +771,41 @@ static void cmd_info(int dev_id, int argc, char **argv)
 
 	printf("\tBD Address:  %s\n", argv[0]);
 
+	ba2oui(&bdaddr, oui);
+	comp = ouitocomp(oui);
+	if (comp) {
+		printf("\tOUI Company: %s (%s)\n", comp, oui);
+		free(comp);
+	}
+
 	if (hci_read_remote_name(dd, &bdaddr, sizeof(name), name, 25000) == 0)
 		printf("\tDevice Name: %s\n", name);
 
 	if (hci_read_remote_version(dd, handle, &version, 20000) == 0) {
+		char *ver = lmp_vertostr(version.lmp_ver);
 		printf("\tLMP Version: %s (0x%x) LMP Subversion: 0x%x\n"
 			"\tManufacturer: %s (%d)\n",
-			lmp_vertostr(version.lmp_ver),
+			ver ? ver : "n/a",
 			version.lmp_ver,
 			version.lmp_subver,
 			bt_compidtostr(version.manufacturer),
 			version.manufacturer);
+		if (ver)
+			bt_free(ver);
 	}
 
 	if (hci_read_remote_features(dd, handle, features, 20000) == 0) {
+		char *tmp = lmp_featurestostr(features, "\t\t", 63);
 		printf("\tFeatures: 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x 0x%2.2x\n%s\n",
 			features[0], features[1], features[2], features[3],
-			features[4], features[5], features[6], features[7],
-			lmp_featurestostr(features, "\t\t", 63));
+			features[4], features[5], features[6], features[7], tmp);
+		bt_free(tmp);
+	}
+
+	if (features[7] & LMP_EXT_FEAT) {
+		if (hci_read_remote_ext_features(dd, handle, 0, &max_page, features, 20000) == 0)
+			if (max_page > 0)
+				printf("\tExtended features: %d pages\n", max_page);
 	}
 
 	if (cc) {
@@ -494,7 +831,7 @@ static char *cmd_help =
 
 static void cmd_cmd(int dev_id, int argc, char **argv)
 {
-	char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
+	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
 	struct hci_filter flt;
 	hci_event_hdr *hdr;
 	int i, opt, len, dd;
@@ -615,9 +952,10 @@ static char *cc_help =
 static void cmd_cc(int dev_id, int argc, char **argv)
 {
 	bdaddr_t bdaddr;
-	int opt, ptype, dd;
 	uint16_t handle;
 	uint8_t role;
+	unsigned int ptype;
+	int dd, opt;
 
 	role = 0x01;
 	ptype = HCI_DM1 | HCI_DM3 | HCI_DM5 | HCI_DH1 | HCI_DH3 | HCI_DH5;
@@ -662,7 +1000,7 @@ static void cmd_cc(int dev_id, int argc, char **argv)
 	}
 
 	if (hci_create_connection(dd, &bdaddr, htobs(ptype),
-					0, role, &handle, 25000) < 0)
+				htobs(0x0000), role, &handle, 25000) < 0)
 		perror("Can't create connection");
 	hci_close_dev(dd);
 }
@@ -716,8 +1054,10 @@ static void cmd_dc(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -851,8 +1191,10 @@ static void cmd_rssi(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -922,8 +1264,10 @@ static void cmd_lq(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -958,7 +1302,8 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
 	bdaddr_t bdaddr;
-	uint8_t type, level;
+	uint8_t type;
+	int8_t level;
 	int opt, dd;
 
 	for_each_opt(opt, tpl_options, NULL) {
@@ -994,8 +1339,10 @@ static void cmd_tpl(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1067,8 +1414,10 @@ static void cmd_afh(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1115,7 +1464,8 @@ static void cmd_cpt(int dev_id, int argc, char **argv)
 	set_conn_ptype_cp cp;
 	evt_conn_ptype_changed rp;
 	bdaddr_t bdaddr;
-	int opt, dd, ptype;
+	unsigned int ptype;
+	int dd, opt;
 
 	for_each_opt(opt, cpt_options, NULL) {
 		switch (opt) {
@@ -1150,8 +1500,10 @@ static void cmd_cpt(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1231,8 +1583,10 @@ static void cmd_lst(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1316,8 +1670,10 @@ static void cmd_auth(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1385,8 +1741,10 @@ static void cmd_enc(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1455,8 +1813,10 @@ static void cmd_key(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1524,8 +1884,10 @@ static void cmd_clkoff(int dev_id, int argc, char **argv)
 	}
 
 	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (!cr) {
+		perror("Can't allocate memory");
+		exit(1);
+	}
 
 	bacpy(&cr->bdaddr, &bdaddr);
 	cr->type = ACL_LINK;
@@ -1554,14 +1916,14 @@ static struct option clock_options[] = {
 
 static char *clock_help =
 	"Usage:\n"
-	"\tclock <bdaddr> [which clock]\n";
+	"\tclock [bdaddr] [which clock]\n";
 
 static void cmd_clock(int dev_id, int argc, char **argv)
 {
 	struct hci_conn_info_req *cr;
 	bdaddr_t bdaddr;
 	uint8_t which;
-	uint32_t clock;
+	uint32_t handle, clock;
 	uint16_t accuracy;
 	int opt, dd;
 
@@ -1575,12 +1937,13 @@ static void cmd_clock(int dev_id, int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	if (argc < 1) {
-		printf(clock_help);
-		return;
-	}
+	if (argc > 0)
+		str2ba(argv[0], &bdaddr);
+	else
+		bacpy(&bdaddr, BDADDR_ANY);
 
-	str2ba(argv[0], &bdaddr);
+	if (!bacmp(&bdaddr, BDADDR_ANY))
+		dev_id = hci_get_route(NULL);
 
 	if (dev_id < 0) {
 		dev_id = hci_for_each_dev(HCI_UP, find_conn, (long) &bdaddr);
@@ -1596,20 +1959,31 @@ static void cmd_clock(int dev_id, int argc, char **argv)
 		exit(1);
 	}
 
-	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
-	if (!cr)
-		return;
+	if (bacmp(&bdaddr, BDADDR_ANY)) {
+		cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+		if (!cr) {
+			perror("Can't allocate memory");
+			exit(1);
+		}
 
-	bacpy(&cr->bdaddr, &bdaddr);
-	cr->type = ACL_LINK;
-	if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
-		perror("Get connection info failed");
-		exit(1);
+		bacpy(&cr->bdaddr, &bdaddr);
+		cr->type = ACL_LINK;
+		if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
+			perror("Get connection info failed");
+			free(cr);
+			exit(1);
+		}
+
+		handle = htobs(cr->conn_info->handle);
+		which = (argc > 1) ? atoi(argv[1]) : 0x01;
+
+		free(cr);
+	} else {
+		handle = 0x00;
+		which = 0x00;
 	}
 
-	which = (argc > 1) ? atoi(argv[1]) : 0x01;
-
-	if (hci_read_clock(dd, htobs(cr->conn_info->handle), which, &clock, &accuracy, 1000) < 0) {
+	if (hci_read_clock(dd, handle, which, &clock, &accuracy, 1000) < 0) {
 		perror("Reading clock failed");
 		exit(1);
 	}
@@ -1620,7 +1994,6 @@ static void cmd_clock(int dev_id, int argc, char **argv)
 	printf("Accuracy: %.2f msec\n", (float) accuracy * 0.3125);
 
 	close(dd);
-	free(cr);
 }
 
 static struct {
