@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: l2ping.c,v 1.8 2005/01/28 17:43:16 holtmann Exp $
+ *  $Id: l2ping.c,v 1.10 2005/03/18 13:49:49 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,7 +50,7 @@
 
 /* Defaults */
 static bdaddr_t bdaddr;
-static int size    = 20;
+static int size    = 44;
 static int ident   = 200;
 static int delay   = 1;
 static int count   = -1;
@@ -193,7 +193,7 @@ static void ping(char *svr)
 			cmd->len = btohs(cmd->len);
 
 			/* Check for our id */
-			if( cmd->ident != id )
+			if (cmd->ident != id)
 				continue;
 
 			/* Check type */
@@ -218,7 +218,7 @@ static void ping(char *svr)
 			if (delay)
 				sleep(delay);
 		} else {
-			printf("no response from %s: id %d\n", svr, id);
+			printf("no response from %s: id %d\n", svr, id - ident);
 		}
 
 		if (++id > 254)
